@@ -40,22 +40,20 @@
 import builtins
 import collections
 import itertools
-import sys
 import re
+import sys
 from typing import Pattern
 
 import astroid
 import astroid.bases
 import astroid.scoped_nodes
+from astroid.arguments import CallSite
 
-from pylint import checkers
-from pylint import exceptions
-from pylint import interfaces
-from pylint.checkers import utils
-from pylint import reporters
-from pylint.checkers.utils import get_node_last_lineno
-from pylint.reporters.ureports import nodes as reporter_nodes
 import pylint.utils as lint_utils
+from pylint import checkers, exceptions, interfaces
+from pylint.checkers import utils
+from pylint.checkers.utils import is_property_setter_or_deleter
+from pylint.reporters.ureports import nodes as reporter_nodes
 
 
 class NamingStyle:
@@ -156,9 +154,6 @@ LITERAL_NODE_TYPES = (astroid.Const, astroid.Dict, astroid.List, astroid.Set)
 UNITTEST_CASE = "unittest.case"
 BUILTINS = builtins.__name__
 TYPE_QNAME = "%s.type" % BUILTINS
-PY33 = sys.version_info >= (3, 3)
-PY3K = sys.version_info >= (3, 0)
-PY35 = sys.version_info >= (3, 5)
 ABC_METACLASSES = {"_py_abc.ABCMeta", "abc.ABCMeta"}  # Python 3.7+,
 
 # Name categories that are always consistent with all naming conventions.
@@ -176,6 +171,7 @@ REVERSED_COMPS = {"<": ">", "<=": ">=", ">": "<", ">=": "<="}
 COMPARISON_OPERATORS = frozenset(("==", "!=", "<", ">", "<=", ">="))
 # List of methods which can be redefined
 REDEFINABLE_METHODS = frozenset(("__module__",))
+TYPING_FORWARD_REF_QNAME = "typing.ForwardRef"
 
 
 def _redefines_import(node):
@@ -319,6 +315,12 @@ def _determine_function_name_type(node, config=None):
     property_classes, property_names = _get_properties(config)
     if not node.is_method():
         return "function"
+
+    if is_property_setter_or_deleter(node):
+        # If the function is decorated using the prop_method.{setter,getter}
+        # form, treat it like an attribute as well.
+        return "attr"
+
     if node.decorators:
         decorators = node.decorators.nodes
     else:
@@ -330,16 +332,9 @@ def _determine_function_name_type(node, config=None):
             isinstance(decorator, astroid.Attribute)
             and decorator.attrname in property_names
         ):
-            infered = utils.safe_infer(decorator)
-            if infered and infered.qname() in property_classes:
+            inferred = utils.safe_infer(decorator)
+            if inferred and inferred.qname() in property_classes:
                 return "attr"
-        # If the function is decorated using the prop_method.{setter,getter}
-        # form, treat it like an attribute as well.
-        elif isinstance(decorator, astroid.Attribute) and decorator.attrname in (
-            "setter",
-            "deleter",
-        ):
-            return "attr"
     return "method"
 
 
@@ -353,7 +348,7 @@ def _has_abstract_methods(node):
     return len(utils.unimplemented_abstract_methods(node)) > 0
 
 
-def report_by_type_stats(sect, stats, old_stats):
+def report_by_type_stats(sect, stats, _):
     """make a report of
 
     * percentage of different types documented
@@ -382,16 +377,11 @@ def report_by_type_stats(sect, stats, old_stats):
     lines = ("type", "number", "old number", "difference", "%documented", "%badname")
     for node_type in ("module", "class", "method", "function"):
         new = stats[node_type]
-        old = old_stats.get(node_type, None)
-        if old is not None:
-            diff_str = reporters.diff_string(old, new)
-        else:
-            old, diff_str = "NC", "NC"
         lines += (
             node_type,
             str(new),
-            str(old),
-            diff_str,
+            "NC",
+            "NC",
             nice_stats[node_type].get("percent_documented", "0"),
             nice_stats[node_type].get("percent_badname", "0"),
         )
@@ -563,7 +553,7 @@ class BasicErrorChecker(_BasicChecker):
             # f(*args) is converted to Call(args=[Starred]), so ignore
             # them for this check.
             return
-        if PY35 and isinstance(
+        if isinstance(
             node.parent, (astroid.List, astroid.Tuple, astroid.Set, astroid.Dict)
         ):
             # PEP 448 unpacking.
@@ -604,19 +594,6 @@ class BasicErrorChecker(_BasicChecker):
                 # Are we returning anything but None from constructors
                 if any(v for v in values if not utils.is_none(v)):
                     self.add_message("return-in-init", node=node)
-        elif node.is_generator():
-            # make sure we don't mix non-None returns and yields
-            if not PY33:
-                for retnode in returns:
-                    if (
-                        isinstance(retnode.value, astroid.Const)
-                        and retnode.value.value is not None
-                    ):
-                        self.add_message(
-                            "return-arg-in-generator",
-                            node=node,
-                            line=retnode.fromlineno,
-                        )
         # Check for duplicate names by clustering args with same name for detailed report
         arg_clusters = collections.defaultdict(list)
         arguments = filter(None, [node.args.args, node.args.kwonlyargs])
@@ -767,12 +744,12 @@ class BasicErrorChecker(_BasicChecker):
         except astroid.InferenceError:
             return
 
-    def _check_inferred_class_is_abstract(self, infered, node):
-        if not isinstance(infered, astroid.ClassDef):
+    def _check_inferred_class_is_abstract(self, inferred, node):
+        if not isinstance(inferred, astroid.ClassDef):
             return
 
         klass = utils.node_frame_class(node)
-        if klass is infered:
+        if klass is inferred:
             # Don't emit the warning if the class is instantiated
             # in its own body or if the call is not an instance
             # creation. If the class is instantiated into its own
@@ -780,20 +757,20 @@ class BasicErrorChecker(_BasicChecker):
             return
 
         # __init__ was called
-        abstract_methods = _has_abstract_methods(infered)
+        abstract_methods = _has_abstract_methods(inferred)
 
         if not abstract_methods:
             return
 
-        metaclass = infered.metaclass()
+        metaclass = inferred.metaclass()
 
         if metaclass is None:
             # Python 3.4 has `abc.ABC`, which won't be detected
             # by ClassNode.metaclass()
-            for ancestor in infered.ancestors():
+            for ancestor in inferred.ancestors():
                 if ancestor.qname() == "abc.ABC":
                     self.add_message(
-                        "abstract-class-instantiated", args=(infered.name,), node=node
+                        "abstract-class-instantiated", args=(inferred.name,), node=node
                     )
                     break
 
@@ -801,7 +778,7 @@ class BasicErrorChecker(_BasicChecker):
 
         if metaclass.qname() in ABC_METACLASSES:
             self.add_message(
-                "abstract-class-instantiated", args=(infered.name,), node=node
+                "abstract-class-instantiated", args=(inferred.name,), node=node
             )
 
     def _check_yield_outside_func(self, node):
@@ -844,7 +821,13 @@ class BasicErrorChecker(_BasicChecker):
     def _check_redefinition(self, redeftype, node):
         """check for redefinition of a function / method / class name"""
         parent_frame = node.parent.frame()
-        defined_self = parent_frame[node.name]
+
+        # Ignore function stubs created for type information
+        redefinitions = parent_frame.locals[node.name]
+        defined_self = next(
+            (local for local in redefinitions if not utils.is_overload_stub(local)),
+            node,
+        )
         if defined_self is not node and not astroid.are_exclusive(node, defined_self):
 
             # Additional checks for methods which are not considered
@@ -855,6 +838,24 @@ class BasicErrorChecker(_BasicChecker):
             ):
                 return
 
+            if utils.is_overload_stub(node):
+                return
+
+            # Check if we have forward references for this node.
+            try:
+                redefinition_index = redefinitions.index(node)
+            except ValueError:
+                pass
+            else:
+                for redefinition in redefinitions[:redefinition_index]:
+                    inferred = utils.safe_infer(redefinition)
+                    if (
+                        inferred
+                        and isinstance(inferred, astroid.Instance)
+                        and inferred.qname() == TYPING_FORWARD_REF_QNAME
+                    ):
+                        return
+
             dummy_variables_rgx = lint_utils.get_global_option(
                 self, "dummy-variables-rgx", default=None
             )
@@ -950,7 +951,7 @@ class BasicChecker(_BasicChecker):
             "re-raised.",
         ),
         "W0199": (
-            "Assert called on a 2-uple. Did you mean 'assert x,y'?",
+            "Assert called on a 2-item-tuple. Did you mean 'assert x,y'?",
             "assert-on-tuple",
             "A call of assert on a tuple will always evaluate to true if "
             "the tuple is not empty, and will always evaluate to false if "
@@ -972,6 +973,22 @@ class BasicChecker(_BasicChecker):
             "uses a constant value for its test. This might not be what "
             "the user intended to do.",
         ),
+        "W0126": (
+            "Using a conditional statement with potentially wrong function or method call due to missing parentheses",
+            "missing-parentheses-for-call-in-test",
+            "Emitted when a conditional statement (If or ternary if) "
+            "seems to wrongly call a function due to missing parentheses",
+        ),
+        "W0127": (
+            "Assigning the same variable %r to itself",
+            "self-assigning-variable",
+            "Emitted when we detect that a variable is assigned to itself",
+        ),
+        "W0128": (
+            "Redeclared variable %r in assignment",
+            "redeclared-assigned-name",
+            "Emitted when we detect that a variable was redeclared in the same assignment.",
+        ),
         "E0111": (
             "The first reversed() argument is not a sequence",
             "bad-reversed-sequence",
@@ -1002,15 +1019,15 @@ class BasicChecker(_BasicChecker):
         self._tryfinallys = []
         self.stats = self.linter.add_stats(module=0, function=0, method=0, class_=0)
 
-    @utils.check_messages("using-constant-test")
+    @utils.check_messages("using-constant-test", "missing-parentheses-for-call-in-test")
     def visit_if(self, node):
         self._check_using_constant_test(node, node.test)
 
-    @utils.check_messages("using-constant-test")
+    @utils.check_messages("using-constant-test", "missing-parentheses-for-call-in-test")
     def visit_ifexp(self, node):
         self._check_using_constant_test(node, node.test)
 
-    @utils.check_messages("using-constant-test")
+    @utils.check_messages("using-constant-test", "missing-parentheses-for-call-in-test")
     def visit_comprehension(self, node):
         if node.ifs:
             for if_test in node.ifs:
@@ -1031,13 +1048,8 @@ class BasicChecker(_BasicChecker):
         structs = (astroid.Dict, astroid.Tuple, astroid.Set)
 
         # These nodes are excepted, since they are not constant
-        # values, requiring a computation to happen. The only type
-        # of node in this list which doesn't have this property is
-        # Attribute, which is excepted because the conditional statement
-        # can be used to verify that the attribute was set inside a class,
-        # which is definitely a valid use case.
+        # values, requiring a computation to happen.
         except_nodes = (
-            astroid.Attribute,
             astroid.Call,
             astroid.BinOp,
             astroid.BoolOp,
@@ -1049,7 +1061,26 @@ class BasicChecker(_BasicChecker):
         if not isinstance(test, except_nodes):
             inferred = utils.safe_infer(test)
 
-        if emit or isinstance(inferred, const_nodes):
+        if emit:
+            self.add_message("using-constant-test", node=node)
+        elif isinstance(inferred, const_nodes):
+            # If the constant node is a FunctionDef or Lambda then
+            #  it may be a illicit function call due to missing parentheses
+            call_inferred = None
+            if isinstance(inferred, astroid.FunctionDef):
+                call_inferred = inferred.infer_call_result()
+            elif isinstance(inferred, astroid.Lambda):
+                call_inferred = inferred.infer_call_result(node)
+            if call_inferred:
+                try:
+                    for inf_call in call_inferred:
+                        if inf_call != astroid.Uninferable:
+                            self.add_message(
+                                "missing-parentheses-for-call-in-test", node=node
+                            )
+                            break
+                except astroid.InferenceError:
+                    pass
             self.add_message("using-constant-test", node=node)
 
     def visit_module(self, _):
@@ -1067,7 +1098,7 @@ class BasicChecker(_BasicChecker):
         "pointless-statement", "pointless-string-statement", "expression-not-assigned"
     )
     def visit_expr(self, node):
-        """check for various kind of statements without effect"""
+        """Check for various kind of statements without effect"""
         expr = node.value
         if isinstance(expr, astroid.Const) and isinstance(expr.value, str):
             # treat string statement in a separated message
@@ -1094,14 +1125,19 @@ class BasicChecker(_BasicChecker):
         # Ignore if this is :
         # * a direct function call
         # * the unique child of a try/except body
-        # * a yieldd statement
+        # * a yield statement
         # * an ellipsis (which can be used on Python 3 instead of pass)
         # warn W0106 if we have any underlying function call (we can't predict
         # side effects), else pointless-statement
-        if isinstance(
-            expr, (astroid.Yield, astroid.Await, astroid.Ellipsis, astroid.Call)
-        ) or (
-            isinstance(node.parent, astroid.TryExcept) and node.parent.body == [node]
+        if (
+            isinstance(
+                expr, (astroid.Yield, astroid.Await, astroid.Ellipsis, astroid.Call)
+            )
+            or (
+                isinstance(node.parent, astroid.TryExcept)
+                and node.parent.body == [node]
+            )
+            or (isinstance(expr, astroid.Const) and expr.value is Ellipsis)
         ):
             return
         if any(expr.nodes_of_class(astroid.Call)):
@@ -1163,13 +1199,12 @@ class BasicChecker(_BasicChecker):
             # return something else (but we don't check that, yet).
             return
 
+        call_site = CallSite.from_call(call)
         ordinary_args = list(node.args.args)
         new_call_args = list(self._filter_vararg(node, call.args))
         if node.args.kwarg:
             if self._has_variadic_argument(call.kwargs, node.args.kwarg):
                 return
-        elif call.kwargs or call.keywords:
-            return
 
         if node.args.vararg:
             if self._has_variadic_argument(call.starargs, node.args.vararg):
@@ -1177,6 +1212,16 @@ class BasicChecker(_BasicChecker):
         elif call.starargs:
             return
 
+        if call.keywords:
+            # Look for additional keyword arguments that are not part
+            # of the lambda's signature
+            lambda_kwargs = {keyword.name for keyword in node.args.defaults}
+            if len(lambda_kwargs) != len(call_site.keyword_arguments):
+                # Different lengths, so probably not identical
+                return
+            if set(call_site.keyword_arguments).difference(lambda_kwargs):
+                return
+
         # The "ordinary" arguments must be in a correspondence such that:
         # ordinary_args[i].name == call.args[i].name.
         if len(ordinary_args) != len(new_call_args):
@@ -1194,7 +1239,7 @@ class BasicChecker(_BasicChecker):
         """check function name, docstring, arguments, redefinition,
         variable names, max locals
         """
-        self.stats[node.is_method() and "method" or "function"] += 1
+        self.stats["method" if node.is_method() else "function"] += 1
         self._check_dangerous_default(node)
 
     visit_asyncfunctiondef = visit_functiondef
@@ -1383,7 +1428,7 @@ class BasicChecker(_BasicChecker):
             if argument is astroid.Uninferable:
                 return
             if argument is None:
-                # Nothing was infered.
+                # Nothing was inferred.
                 # Try to see if we have iter().
                 if isinstance(node.args[0], astroid.Call):
                     try:
@@ -1434,31 +1479,85 @@ class BasicChecker(_BasicChecker):
 
     @utils.check_messages("confusing-with-statement")
     def visit_with(self, node):
-        if not PY3K:
-            # in Python 2 a "with" statement with multiple managers coresponds
-            # to multiple nested AST "With" nodes
-            pairs = []
-            parent_node = node.parent
-            if isinstance(parent_node, astroid.With):
-                # we only care about the direct parent, since this method
-                # gets called for each with node anyway
-                pairs.extend(parent_node.items)
-            pairs.extend(node.items)
-        else:
-            # in PY3K a "with" statement with multiple managers coresponds
-            # to one AST "With" node with multiple items
-            pairs = node.items
+        # a "with" statement with multiple managers coresponds
+        # to one AST "With" node with multiple items
+        pairs = node.items
         if pairs:
             for prev_pair, pair in zip(pairs, pairs[1:]):
                 if isinstance(prev_pair[1], astroid.AssignName) and (
                     pair[1] is None and not isinstance(pair[0], astroid.Call)
                 ):
-                    # don't emit a message if the second is a function call
-                    # there's no way that can be mistaken for a name assignment
-                    if PY3K or node.lineno == node.parent.lineno:
-                        # if the line number doesn't match
-                        # we assume it's a nested "with"
-                        self.add_message("confusing-with-statement", node=node)
+                    # Don't emit a message if the second is a function call
+                    # there's no way that can be mistaken for a name assignment.
+                    # If the line number doesn't match
+                    # we assume it's a nested "with".
+                    self.add_message("confusing-with-statement", node=node)
+
+    def _check_self_assigning_variable(self, node):
+        # Detect assigning to the same variable.
+
+        scope = node.scope()
+        scope_locals = scope.locals
+
+        rhs_names = []
+        targets = node.targets
+        if isinstance(targets[0], astroid.Tuple):
+            if len(targets) != 1:
+                # A complex assignment, so bail out early.
+                return
+            targets = targets[0].elts
+
+        if isinstance(node.value, astroid.Name):
+            if len(targets) != 1:
+                return
+            rhs_names = [node.value]
+        elif isinstance(node.value, astroid.Tuple):
+            rhs_count = len(node.value.elts)
+            if len(targets) != rhs_count or rhs_count == 1:
+                return
+            rhs_names = node.value.elts
+
+        for target, lhs_name in zip(targets, rhs_names):
+            if not isinstance(lhs_name, astroid.Name):
+                continue
+            if not isinstance(target, astroid.AssignName):
+                continue
+            if isinstance(scope, astroid.ClassDef) and target.name in scope_locals:
+                # Check that the scope is different than a class level, which is usually
+                # a pattern to expose module level attributes as class level ones.
+                continue
+            if target.name == lhs_name.name:
+                self.add_message(
+                    "self-assigning-variable", args=(target.name,), node=target
+                )
+
+    def _check_redeclared_assign_name(self, targets):
+        for target in targets:
+            if not isinstance(target, astroid.Tuple):
+                continue
+
+            found_names = []
+            for element in target.elts:
+                if isinstance(element, astroid.Tuple):
+                    self._check_redeclared_assign_name([element])
+                elif isinstance(element, astroid.AssignName) and element.name != "_":
+                    found_names.append(element.name)
+
+            names = collections.Counter(found_names)
+            for name, count in names.most_common():
+                if count > 1:
+                    self.add_message(
+                        "redeclared-assigned-name", args=(name,), node=target
+                    )
+
+    @utils.check_messages("self-assigning-variable", "redeclared-assigned-name")
+    def visit_assign(self, node):
+        self._check_self_assigning_variable(node)
+        self._check_redeclared_assign_name(node.targets)
+
+    @utils.check_messages("redeclared-assigned-name")
+    def visit_for(self, node):
+        self._check_redeclared_assign_name([node.target])
 
 
 KNOWN_NAME_TYPES = {
@@ -1837,18 +1936,34 @@ class NameChecker(_BasicChecker):
 
 class DocStringChecker(_BasicChecker):
     msgs = {
-        "C0111": (
-            "Missing %s docstring",  # W0131
-            "missing-docstring",
-            "Used when a module, function, class or method has no docstring."
-            "Some special methods like __init__ doesn't necessary require a "
-            "docstring.",
-        ),
         "C0112": (
-            "Empty %s docstring",  # W0132
+            "Empty %s docstring",
             "empty-docstring",
             "Used when a module, function, class or method has an empty "
             "docstring (it would be too easy ;).",
+            {"old_names": [("W0132", "old-empty-docstring")]},
+        ),
+        "C0114": (
+            "Missing module docstring",
+            "missing-module-docstring",
+            "Used when a module has no docstring."
+            "Empty modules do not require a docstring.",
+            {"old_names": [("C0111", "missing-docstring")]},
+        ),
+        "C0115": (
+            "Missing class docstring",
+            "missing-class-docstring",
+            "Used when a class has no docstring."
+            "Even an empty class must have a docstring.",
+            {"old_names": [("C0111", "missing-docstring")]},
+        ),
+        "C0116": (
+            "Missing function or method docstring",
+            "missing-function-docstring",
+            "Used when a function or method has no docstring."
+            "Some special methods like __init__ do not require a "
+            "docstring.",
+            {"old_names": [("C0111", "missing-docstring")]},
         ),
     }
     options = (
@@ -1894,19 +2009,11 @@ class DocStringChecker(_BasicChecker):
         if self.config.no_docstring_rgx.match(node.name) is None:
             self._check_docstring("class", node)
 
-    @staticmethod
-    def _is_setter_or_deleter(node):
-        names = {"setter", "deleter"}
-        for decorator in node.decorators.nodes:
-            if isinstance(decorator, astroid.Attribute) and decorator.attrname in names:
-                return True
-        return False
-
     @utils.check_messages("missing-docstring", "empty-docstring")
     def visit_functiondef(self, node):
         if self.config.no_docstring_rgx.match(node.name) is None:
             ftype = "method" if node.is_method() else "function"
-            if node.decorators and self._is_setter_or_deleter(node):
+            if is_property_setter_or_deleter(node):
                 return
 
             if isinstance(node.parent.frame(), astroid.ClassDef):
@@ -1941,7 +2048,7 @@ class DocStringChecker(_BasicChecker):
         if docstring is None:
             if not report_missing:
                 return
-            lines = get_node_last_lineno(node) - node.lineno
+            lines = utils.get_node_last_lineno(node) - node.lineno
 
             if node_type == "module" and not lines:
                 # If the module has no body, there's no reason
@@ -1962,14 +2069,18 @@ class DocStringChecker(_BasicChecker):
                 if isinstance(func, astroid.BoundMethod) and isinstance(
                     func.bound, astroid.Instance
                 ):
-                    # Strings in Python 3, others in Python 2.
-                    if PY3K and func.bound.name == "str":
+                    # Strings.
+                    if func.bound.name == "str":
                         return
                     if func.bound.name in ("str", "unicode", "bytes"):
                         return
-            self.add_message(
-                "missing-docstring", node=node, args=(node_type,), confidence=confidence
-            )
+            if node_type == "module":
+                message = "missing-module-docstring"
+            elif node_type == "class":
+                message = "missing-class-docstring"
+            else:
+                message = "missing-function-docstring"
+            self.add_message(message, node=node, confidence=confidence)
         elif not docstring.strip():
             self.stats["undocumented_" + node_type] += 1
             self.add_message(
@@ -2034,7 +2145,7 @@ class ComparisonChecker(_BasicChecker):
             "Python is to use isinstance(x, Y) rather than "
             "type(x) == Y, type(x) is Y. Though there are unusual "
             "situations where these give different results.",
-            {"old_names": [("W0154", "unidiomatic-typecheck")]},
+            {"old_names": [("W0154", "old-unidiomatic-typecheck")]},
         ),
         "R0123": (
             "Comparison to literal",
