@@ -18,15 +18,13 @@
 
 """Check source code is ascii only or has an encoding declaration (PEP 263)"""
 
-# pylint: disable=W0511
-
+import re
 import tokenize
 
-import re
-
-from pylint.interfaces import IRawChecker, ITokenChecker
 from pylint.checkers import BaseChecker
-from pylint.utils import OPTION_RGX, MessagesHandlerMixIn
+from pylint.constants import OPTION_RGX
+from pylint.interfaces import IRawChecker, ITokenChecker
+from pylint.message import MessagesHandlerMixIn
 
 
 class ByIdManagedMessagesChecker(BaseChecker):
@@ -80,14 +78,7 @@ class EncodingChecker(BaseChecker):
             "%s",
             "fixme",
             "Used when a warning note as FIXME or XXX is detected.",
-        ),
-        "W0512": (
-            'Cannot decode using encoding "%s", unexpected byte at position %d',
-            "invalid-encoded-data",
-            "Used when a source line cannot be decoded using the specified "
-            "source file encoding.",
-            {"maxversion": (3, 0)},
-        ),
+        )
     }
 
     options = (
@@ -114,11 +105,9 @@ class EncodingChecker(BaseChecker):
     def _check_encoding(self, lineno, line, file_encoding):
         try:
             return line.decode(file_encoding)
-        except UnicodeDecodeError as ex:
-            self.add_message(
-                "invalid-encoded-data", line=lineno, args=(file_encoding, ex.args[2])
-            )
-        except LookupError as ex:
+        except UnicodeDecodeError:
+            pass
+        except LookupError:
             if line.startswith("#") and "coding" in line and file_encoding in line:
                 self.add_message(
                     "syntax-error",
@@ -160,7 +149,7 @@ class EncodingChecker(BaseChecker):
                     self.add_message(
                         "bad-inline-option",
                         args=disable_option_match.group(1).strip(),
-                        line=comment.string,
+                        line=comment.start[0],
                     )
                     continue
 
