@@ -24,23 +24,18 @@
 # For details: https://github.com/PyCQA/pylint/blob/master/COPYING
 
 """Check Python 2 code for Python 2/3 source-compatible issues."""
-from __future__ import absolute_import, print_function
-
-from collections import namedtuple
 import re
-import sys
 import tokenize
-from typing import FrozenSet
+from collections import namedtuple
 
 import astroid
 from astroid import bases
 
 from pylint import checkers, interfaces
-from pylint.checkers.utils import node_ignores_exception, find_try_except_wrapper_node
-from pylint.interfaces import INFERENCE_FAILURE, INFERENCE
-from pylint.utils import WarningScope
 from pylint.checkers import utils
-
+from pylint.checkers.utils import find_try_except_wrapper_node, node_ignores_exception
+from pylint.constants import WarningScope
+from pylint.interfaces import INFERENCE, INFERENCE_FAILURE
 
 _ZERO = re.compile("^0+$")
 
@@ -114,8 +109,7 @@ def _in_iterating_context(node):
     # value.
     elif isinstance(parent, astroid.Call):
         if isinstance(parent.func, astroid.Name):
-            parent_scope = parent.func.lookup(parent.func.name)[0]
-            if _is_builtin(parent_scope) and parent.func.name in _ACCEPTS_ITERATOR:
+            if parent.func.name in _ACCEPTS_ITERATOR:
                 return True
         elif isinstance(parent.func, astroid.Attribute):
             if parent.func.attrname in ATTRIBUTES_ACCEPTS_ITERATOR:
@@ -190,7 +184,7 @@ class Python3Checker(checkers.BaseChecker):
             "Python3 will not allow implicit unpacking of "
             "exceptions in except clauses. "
             "See http://www.python.org/dev/peps/pep-3110/",
-            {"old_names": [("W0712", "unpacking-in-except")]},
+            {"old_names": [("W0712", "old-unpacking-in-except")]},
         ),
         "E1604": (
             "Use raise ErrorClass(args) instead of raise ErrorClass, args.",
@@ -198,14 +192,14 @@ class Python3Checker(checkers.BaseChecker):
             "Used when the alternate raise syntax "
             "'raise foo, bar' is used "
             "instead of 'raise foo(bar)'.",
-            {"old_names": [("W0121", "old-raise-syntax")]},
+            {"old_names": [("W0121", "old-old-raise-syntax")]},
         ),
         "E1605": (
             "Use of the `` operator",
             "backtick",
             'Used when the deprecated "``" (backtick) operator is used '
             "instead  of the str() function.",
-            {"scope": WarningScope.NODE, "old_names": [("W0333", "backtick")]},
+            {"scope": WarningScope.NODE, "old_names": [("W0333", "old-backtick")]},
         ),
         "E1609": (
             "Import * only allowed at module level",
@@ -359,14 +353,14 @@ class Python3Checker(checkers.BaseChecker):
             "indexing-exception",
             "Indexing exceptions will not work on Python 3. Use "
             "`exception.args[index]` instead.",
-            {"old_names": [("W0713", "indexing-exception")]},
+            {"old_names": [("W0713", "old-indexing-exception")]},
         ),
         "W1625": (
             "Raising a string exception",
             "raising-string",
             "Used when a string exception is raised. This will not "
             "work on Python 3.",
-            {"old_names": [("W0701", "raising-string")]},
+            {"old_names": [("W0701", "old-raising-string")]},
         ),
         "W1626": (
             "reload built-in referenced",
@@ -438,7 +432,7 @@ class Python3Checker(checkers.BaseChecker):
             "range built-in referenced when not iterating",
             "range-builtin-not-iterating",
             "Used when the range built-in is referenced in a non-iterating "
-            "context (returns an iterator in Python 3)",
+            "context (returns a range in Python 3)",
         ),
         "W1639": (
             "filter built-in referenced when not iterating",
@@ -879,23 +873,18 @@ class Python3Checker(checkers.BaseChecker):
         "deprecated-sys-function": {"sys": frozenset({"exc_clear"})},
     }
 
-    if (3, 4) <= sys.version_info < (3, 4, 4):
-        # Python 3.4.0 -> 3.4.3 has a bug which breaks `repr_tree()`:
-        # https://bugs.python.org/issue23572
-        _python_2_tests = frozenset()  # type: FrozenSet[str]
-    else:
-        _python_2_tests = frozenset(
-            [
-                astroid.extract_node(x).repr_tree()
-                for x in [
-                    "sys.version_info[0] == 2",
-                    "sys.version_info[0] < 3",
-                    "sys.version_info == (2, 7)",
-                    "sys.version_info <= (2, 7)",
-                    "sys.version_info < (3, 0)",
-                ]
+    _python_2_tests = frozenset(
+        [
+            astroid.extract_node(x).repr_tree()
+            for x in [
+                "sys.version_info[0] == 2",
+                "sys.version_info[0] < 3",
+                "sys.version_info == (2, 7)",
+                "sys.version_info <= (2, 7)",
+                "sys.version_info < (3, 0)",
             ]
-        )
+        ]
+    )
 
     def __init__(self, *args, **kwargs):
         self._future_division = False
@@ -1069,7 +1058,18 @@ class Python3Checker(checkers.BaseChecker):
     def visit_binop(self, node):
         if not self._future_division and node.op == "/":
             for arg in (node.left, node.right):
-                if isinstance(arg, astroid.Const) and isinstance(arg.value, float):
+                inferred = utils.safe_infer(arg)
+                # If we can infer the object and that object is not an int, bail out.
+                if inferred and not (
+                    (
+                        isinstance(inferred, astroid.Const)
+                        and isinstance(inferred.value, int)
+                    )
+                    or (
+                        isinstance(inferred, astroid.Instance)
+                        and inferred.name == "int"
+                    )
+                ):
                     break
             else:
                 self.add_message("old-division", node=node)
@@ -1359,7 +1359,7 @@ class Python3TokenChecker(checkers.BaseTokenChecker):
             "old-ne-operator",
             'Used when the deprecated "<>" operator is used instead '
             'of "!=". This is removed in Python 3.',
-            {"maxversion": (3, 0), "old_names": [("W0331", "old-ne-operator")]},
+            {"maxversion": (3, 0), "old_names": [("W0331", "old-old-ne-operator")]},
         ),
         "E1608": (
             "Use of old octal literal",
