@@ -22,39 +22,90 @@
 
 """Looks for code which can be refactored."""
 import builtins
-from functools import reduce
-
 import collections
 import itertools
 import tokenize
+from functools import reduce
 
 import astroid
 from astroid import decorators
 
-from pylint import interfaces
-from pylint import checkers
+from pylint import checkers, interfaces
 from pylint import utils as lint_utils
 from pylint.checkers import utils
 
-
 KNOWN_INFINITE_ITERATORS = {"itertools.count"}
+BUILTIN_EXIT_FUNCS = frozenset(("quit", "exit"))
 
 
-def _all_elements_are_true(gen):
-    values = list(gen)
-    return values and all(values)
+def _if_statement_is_always_returning(if_node, returning_node_class):
+    for node in if_node.body:
+        if isinstance(node, returning_node_class):
+            return True
+    return False
 
 
-def _if_statement_is_always_returning(if_node):
-    def _has_return_node(elems, scope):
-        for node in elems:
-            if isinstance(node, astroid.If) and node.orelse:
-                yield _if_statement_is_always_returning(node)
-            if isinstance(node, astroid.Return):
-                yield node.scope() is scope
+def _is_len_call(node):
+    """Checks if node is len(SOMETHING)."""
+    return (
+        isinstance(node, astroid.Call)
+        and isinstance(node.func, astroid.Name)
+        and node.func.name == "len"
+    )
 
-    scope = if_node.scope()
-    return _all_elements_are_true(_has_return_node(if_node.body, scope=scope))
+
+def _is_constant_zero(node):
+    return isinstance(node, astroid.Const) and node.value == 0
+
+
+def _node_is_test_condition(node):
+    """ Checks if node is an if, while, assert or if expression statement."""
+    return isinstance(node, (astroid.If, astroid.While, astroid.Assert, astroid.IfExp))
+
+
+def _is_trailing_comma(tokens, index):
+    """Check if the given token is a trailing comma
+
+    :param tokens: Sequence of modules tokens
+    :type tokens: list[tokenize.TokenInfo]
+    :param int index: Index of token under check in tokens
+    :returns: True if the token is a comma which trails an expression
+    :rtype: bool
+    """
+    token = tokens[index]
+    if token.exact_type != tokenize.COMMA:
+        return False
+    # Must have remaining tokens on the same line such as NEWLINE
+    left_tokens = itertools.islice(tokens, index + 1, None)
+    same_line_remaining_tokens = list(
+        itertools.takewhile(
+            lambda other_token, _token=token: other_token.start[0] == _token.start[0],
+            left_tokens,
+        )
+    )
+    # Note: If the newline is tokenize.NEWLINE and not tokenize.NL
+    # then the newline denotes the end of expression
+    is_last_element = all(
+        other_token.type in (tokenize.NEWLINE, tokenize.COMMENT)
+        for other_token in same_line_remaining_tokens
+    )
+    if not same_line_remaining_tokens or not is_last_element:
+        return False
+
+    def get_curline_index_start():
+        """Get the index denoting the start of the current line"""
+        for subindex, token in enumerate(reversed(tokens[:index])):
+            # See Lib/tokenize.py and Lib/token.py in cpython for more info
+            if token.type in (tokenize.NEWLINE, tokenize.NL):
+                return index - subindex
+        return 0
+
+    curline_start = get_curline_index_start()
+    expected_tokens = {"return", "yield"}
+    for prevtoken in tokens[curline_start:index]:
+        if "=" in prevtoken.string or prevtoken.string in expected_tokens:
+            return True
+    return False
 
 
 class RefactoringChecker(checkers.BaseTokenChecker):
@@ -91,13 +142,13 @@ class RefactoringChecker(checkers.BaseTokenChecker):
             "Used when a function or a method has too many nested "
             "blocks. This makes the code less understandable and "
             "maintainable.",
-            {"old_names": [("R0101", "too-many-nested-blocks")]},
+            {"old_names": [("R0101", "old-too-many-nested-blocks")]},
         ),
         "R1703": (
             "The if statement can be replaced with %s",
             "simplifiable-if-statement",
             "Used when an if statement can be replaced with 'bool(test)'. ",
-            {"old_names": [("R0102", "simplifiable-if-statement")]},
+            {"old_names": [("R0102", "old-simplifiable-if-statement")]},
         ),
         "R1704": (
             "Redefining argument with the local name %r",
@@ -187,6 +238,8 @@ class RefactoringChecker(checkers.BaseTokenChecker):
         "R1717": (
             "Consider using a dictionary comprehension",
             "consider-using-dict-comprehension",
+            "Emitted when we detect the creation of a dictionary "
+            "using the dict() callable and a transient list. "
             "Although there is nothing syntactically wrong with this code, "
             "it is hard to read and can be simplified to a dict comprehension."
             "Also it is faster since you don't need to create another "
@@ -205,6 +258,45 @@ class RefactoringChecker(checkers.BaseTokenChecker):
             "simplifiable-if-expression",
             "Used when an if expression can be replaced with 'bool(test)'. ",
         ),
+        "R1720": (
+            'Unnecessary "%s" after "raise"',
+            "no-else-raise",
+            "Used in order to highlight an unnecessary block of "
+            "code following an if containing a raise statement. "
+            "As such, it will warn when it encounters an else "
+            "following a chain of ifs, all of them containing a "
+            "raise statement.",
+        ),
+        "R1721": (
+            "Unnecessary use of a comprehension",
+            "unnecessary-comprehension",
+            "Instead of using an identitiy comprehension, "
+            "consider using the list, dict or set constructor. "
+            "It is faster and simpler.",
+        ),
+        "R1722": (
+            "Consider using sys.exit()",
+            "consider-using-sys-exit",
+            "Instead of using exit() or quit(), consider using the sys.exit().",
+        ),
+        "R1723": (
+            'Unnecessary "%s" after "break"',
+            "no-else-break",
+            "Used in order to highlight an unnecessary block of "
+            "code following an if containing a break statement. "
+            "As such, it will warn when it encounters an else "
+            "following a chain of ifs, all of them containing a "
+            "break statement.",
+        ),
+        "R1724": (
+            'Unnecessary "%s" after "continue"',
+            "no-else-continue",
+            "Used in order to highlight an unnecessary block of "
+            "code following an if containing a continue statement. "
+            "As such, it will warn when it encounters an else "
+            "following a chain of ifs, all of them containing a "
+            "continue statement.",
+        ),
     }
     options = (
         (
@@ -356,7 +448,7 @@ class RefactoringChecker(checkers.BaseTokenChecker):
                 # tokens[index][2] is the actual position and also is
                 # reported by IronPython.
                 self._elifs.extend([tokens[index][2], tokens[index + 1][2]])
-            elif is_trailing_comma(tokens, index):
+            elif _is_trailing_comma(tokens, index):
                 if self.linter.is_message_enabled("trailing-comma-tuple"):
                     self.add_message("trailing-comma-tuple", line=token.start[0])
 
@@ -411,18 +503,42 @@ class RefactoringChecker(checkers.BaseTokenChecker):
             for name in names.nodes_of_class(astroid.AssignName):
                 self._check_redefined_argument_from_local(name)
 
-    def _check_superfluous_else_return(self, node):
+    def _check_superfluous_else(self, node, msg_id, returning_node_class):
         if not node.orelse:
             # Not interested in if statements without else.
             return
 
-        if _if_statement_is_always_returning(node) and not self._is_actual_elif(node):
-            orelse = node.orelse and node.orelse[0]
+        if self._is_actual_elif(node):
+            # Not interested in elif nodes; only if
+            return
+
+        if _if_statement_is_always_returning(node, returning_node_class):
+            orelse = node.orelse[0]
             followed_by_elif = (orelse.lineno, orelse.col_offset) in self._elifs
             self.add_message(
-                "no-else-return", node=node, args="elif" if followed_by_elif else "else"
+                msg_id, node=node, args="elif" if followed_by_elif else "else"
             )
 
+    def _check_superfluous_else_return(self, node):
+        return self._check_superfluous_else(
+            node, msg_id="no-else-return", returning_node_class=astroid.Return
+        )
+
+    def _check_superfluous_else_raise(self, node):
+        return self._check_superfluous_else(
+            node, msg_id="no-else-raise", returning_node_class=astroid.Raise
+        )
+
+    def _check_superfluous_else_break(self, node):
+        return self._check_superfluous_else(
+            node, msg_id="no-else-break", returning_node_class=astroid.Break
+        )
+
+    def _check_superfluous_else_continue(self, node):
+        return self._check_superfluous_else(
+            node, msg_id="no-else-continue", returning_node_class=astroid.Continue
+        )
+
     def _check_consider_get(self, node):
         def type_and_name_are_equal(node_a, node_b):
             for _type in [astroid.Name, astroid.AssignName]:
@@ -462,12 +578,18 @@ class RefactoringChecker(checkers.BaseTokenChecker):
         "too-many-nested-blocks",
         "simplifiable-if-statement",
         "no-else-return",
+        "no-else-raise",
+        "no-else-break",
+        "no-else-continue",
         "consider-using-get",
     )
     def visit_if(self, node):
         self._check_simplifiable_if(node)
         self._check_nested_blocks(node)
         self._check_superfluous_else_return(node)
+        self._check_superfluous_else_raise(node)
+        self._check_superfluous_else_break(node)
+        self._check_superfluous_else_continue(node)
         self._check_consider_get(node)
 
     @utils.check_messages("simplifiable-if-expression")
@@ -542,20 +664,45 @@ class RefactoringChecker(checkers.BaseTokenChecker):
         if (
             isinstance(node.func, astroid.Name)
             and node.args
-            and node.func.name in {"dict", "set"}
             and isinstance(node.args[0], astroid.ListComp)
         ):
-            message_name = "consider-using-{}-comprehension".format(node.func.name)
-            self.add_message(message_name, node=node)
+            if node.func.name == "dict" and not isinstance(
+                node.args[0].elt, astroid.Call
+            ):
+                message_name = "consider-using-dict-comprehension"
+                self.add_message(message_name, node=node)
+            elif node.func.name == "set":
+                message_name = "consider-using-set-comprehension"
+                self.add_message(message_name, node=node)
 
     @utils.check_messages(
         "stop-iteration-return",
         "consider-using-dict-comprehension",
         "consider-using-set-comprehension",
+        "consider-using-sys-exit",
     )
     def visit_call(self, node):
         self._check_raising_stopiteration_in_generator_next_call(node)
         self._check_consider_using_comprehension_constructor(node)
+        self._check_quit_exit_call(node)
+
+    @staticmethod
+    def _has_exit_in_scope(scope):
+        exit_func = scope.locals.get("exit")
+        return bool(
+            exit_func and isinstance(exit_func[0], (astroid.ImportFrom, astroid.Import))
+        )
+
+    def _check_quit_exit_call(self, node):
+
+        if isinstance(node.func, astroid.Name) and node.func.name in BUILTIN_EXIT_FUNCS:
+            # If we have `exit` imported from `sys` in the current or global scope, exempt this instance.
+            local_scope = node.scope()
+            if self._has_exit_in_scope(local_scope) or self._has_exit_in_scope(
+                node.root()
+            ):
+                return
+            self.add_message("consider-using-sys-exit", node=node)
 
     def _check_raising_stopiteration_in_generator_next_call(self, node):
         """Check if a StopIteration exception is raised by the call to next function
@@ -729,49 +876,52 @@ class RefactoringChecker(checkers.BaseTokenChecker):
         """Check if there is any chained comparison in the expression.
 
         Add a refactoring message if a boolOp contains comparison like a < b and b < c,
-        which can be chained as  a < b < c.
+        which can be chained as a < b < c.
+
+        Care is taken to avoid simplifying a < b < c and b < d.
         """
-        if (
-            node.op != "and"
-            or len(node.values) < 2
-            or not all(isinstance(value, astroid.Compare) for value in node.values)
-        ):
+        if node.op != "and" or len(node.values) < 2:
             return
 
-        def _find_lower_upper_bounds(comparison_node, lower_bounds, upper_bounds):
-            operator = comparison_node.ops[0][0]
-            left_operand, right_operand = (
-                comparison_node.left,
-                comparison_node.ops[0][1],
-            )
-            for operand in (left_operand, right_operand):
-                value = None
-                if isinstance(operand, astroid.Name):
-                    value = operand.name
-                elif isinstance(operand, astroid.Const):
-                    value = operand.value
+        def _find_lower_upper_bounds(comparison_node, uses):
+            left_operand = comparison_node.left
+            for operator, right_operand in comparison_node.ops:
+                for operand in (left_operand, right_operand):
+                    value = None
+                    if isinstance(operand, astroid.Name):
+                        value = operand.name
+                    elif isinstance(operand, astroid.Const):
+                        value = operand.value
 
-                if value is None:
-                    continue
+                    if value is None:
+                        continue
 
-                if operator in ("<", "<="):
-                    if operand is left_operand:
-                        lower_bounds.append(value)
-                    else:
-                        upper_bounds.append(value)
-                elif operator in (">", ">="):
-                    if operand is left_operand:
-                        upper_bounds.append(value)
-                    else:
-                        lower_bounds.append(value)
+                    if operator in ("<", "<="):
+                        if operand is left_operand:
+                            uses[value]["lower_bound"].add(comparison_node)
+                        elif operand is right_operand:
+                            uses[value]["upper_bound"].add(comparison_node)
+                    elif operator in (">", ">="):
+                        if operand is left_operand:
+                            uses[value]["upper_bound"].add(comparison_node)
+                        elif operand is right_operand:
+                            uses[value]["lower_bound"].add(comparison_node)
+                left_operand = right_operand
 
-        lower_bounds = []
-        upper_bounds = []
+        uses = collections.defaultdict(
+            lambda: {"lower_bound": set(), "upper_bound": set()}
+        )
         for comparison_node in node.values:
-            _find_lower_upper_bounds(comparison_node, lower_bounds, upper_bounds)
+            if isinstance(comparison_node, astroid.Compare):
+                _find_lower_upper_bounds(comparison_node, uses)
 
-        if set(lower_bounds).intersection(upper_bounds):
-            self.add_message("chained-comparison", node=node)
+        for _, bounds in uses.items():
+            num_shared = len(bounds["lower_bound"].intersection(bounds["upper_bound"]))
+            num_lower_bounds = len(bounds["lower_bound"])
+            num_upper_bounds = len(bounds["upper_bound"])
+            if num_shared < num_lower_bounds and num_shared < num_upper_bounds:
+                self.add_message("chained-comparison", node=node)
+                break
 
     @utils.check_messages(
         "consider-merging-isinstance", "consider-using-in", "chained-comparison"
@@ -879,6 +1029,58 @@ class RefactoringChecker(checkers.BaseTokenChecker):
     def visit_augassign(self, node):
         self._check_consider_using_join(node)
 
+    @utils.check_messages("unnecessary-comprehension")
+    def visit_comprehension(self, node):
+        self._check_unnecessary_comprehension(node)
+
+    def _check_unnecessary_comprehension(self, node):
+        if (
+            isinstance(node.parent, astroid.GeneratorExp)
+            or len(node.ifs) != 0
+            or len(node.parent.generators) != 1
+            or node.is_async
+        ):
+            return
+
+        if (
+            isinstance(node.parent, astroid.DictComp)
+            and isinstance(node.parent.key, astroid.Name)
+            and isinstance(node.parent.value, astroid.Name)
+            and isinstance(node.target, astroid.Tuple)
+            and all(isinstance(elt, astroid.AssignName) for elt in node.target.elts)
+        ):
+            expr_list = [node.parent.key.name, node.parent.value.name]
+            target_list = [elt.name for elt in node.target.elts]
+
+        elif isinstance(node.parent, (astroid.ListComp, astroid.SetComp)):
+            expr = node.parent.elt
+            if isinstance(expr, astroid.Name):
+                expr_list = expr.name
+            elif isinstance(expr, astroid.Tuple):
+                if any(not isinstance(elt, astroid.Name) for elt in expr.elts):
+                    return
+                expr_list = [elt.name for elt in expr.elts]
+            else:
+                expr_list = []
+            target = node.parent.generators[0].target
+            target_list = (
+                target.name
+                if isinstance(target, astroid.AssignName)
+                else (
+                    [
+                        elt.name
+                        for elt in target.elts
+                        if isinstance(elt, astroid.AssignName)
+                    ]
+                    if isinstance(target, astroid.Tuple)
+                    else []
+                )
+            )
+        else:
+            return
+        if expr_list == target_list != []:
+            self.add_message("unnecessary-comprehension", node=node)
+
     @staticmethod
     def _is_and_or_ternary(node):
         """
@@ -1072,12 +1274,17 @@ class RecommandationChecker(checkers.BaseChecker):
 
     @utils.check_messages("consider-iterating-dictionary")
     def visit_call(self, node):
+        if not isinstance(node.func, astroid.Attribute):
+            return
+        if node.func.attrname != "keys":
+            return
+        if not isinstance(node.parent, (astroid.For, astroid.Comprehension)):
+            return
+
         inferred = utils.safe_infer(node.func)
-        if not inferred:
-            return
-        if not isinstance(inferred, astroid.BoundMethod):
-            return
-        if not isinstance(inferred.bound, astroid.Dict) or inferred.name != "keys":
+        if not isinstance(inferred, astroid.BoundMethod) or not isinstance(
+            inferred.bound, astroid.Dict
+        ):
             return
 
         if isinstance(node.parent, (astroid.For, astroid.Comprehension)):
@@ -1158,7 +1365,7 @@ class NotChecker(checkers.BaseChecker):
             "Used when a boolean expression contains an unneeded negation.",
         )
     }
-    name = "basic"
+    name = "refactoring"
     reverse_op = {
         "<": ">=",
         "<=": ">",
@@ -1221,28 +1428,6 @@ class NotChecker(checkers.BaseChecker):
             )
 
 
-def _is_len_call(node):
-    """Checks if node is len(SOMETHING)."""
-    return (
-        isinstance(node, astroid.Call)
-        and isinstance(node.func, astroid.Name)
-        and node.func.name == "len"
-    )
-
-
-def _is_constant_zero(node):
-    return isinstance(node, astroid.Const) and node.value == 0
-
-
-def _has_constant_value(node, value):
-    return isinstance(node, astroid.Const) and node.value == value
-
-
-def _node_is_test_condition(node):
-    """ Checks if node is an if, while, assert or if expression statement."""
-    return isinstance(node, (astroid.If, astroid.While, astroid.Assert, astroid.IfExp))
-
-
 class LenChecker(checkers.BaseChecker):
     """Checks for incorrect usage of len() inside conditions.
     Pep8 states:
@@ -1257,25 +1442,27 @@ class LenChecker(checkers.BaseChecker):
     Problems detected:
     * if len(sequence):
     * if not len(sequence):
-    * if len(sequence) == 0:
-    * if len(sequence) != 0:
-    * if len(sequence) > 0:
-    * if len(sequence) < 1:
-    * if len(sequence) <= 0:
+    * elif len(sequence):
+    * elif not len(sequence):
+    * while len(sequence):
+    * while not len(sequence):
+    * assert len(sequence):
+    * assert not len(sequence):
     """
 
     __implements__ = (interfaces.IAstroidChecker,)
 
     # configuration section name
-    name = "len"
+    name = "refactoring"
     msgs = {
         "C1801": (
-            "Do not use `len(SEQUENCE)` to determine if a sequence is empty",
+            "Do not use `len(SEQUENCE)` without comparison to determine if a sequence is empty",
             "len-as-condition",
-            "Used when Pylint detects that len(sequence) is being used inside "
-            "a condition to determine if a sequence is empty. Instead of "
-            "comparing the length to 0, rely on the fact that empty sequences "
-            "are false.",
+            "Used when Pylint detects that len(sequence) is being used "
+            "without explicit comparison inside a condition to determine if a sequence is empty. "
+            "Instead of coercing the length to a boolean, either "
+            "rely on the fact that empty sequences are false or "
+            "compare the length against a scalar.",
         )
     }
 
@@ -1314,109 +1501,6 @@ class LenChecker(checkers.BaseChecker):
         ):
             self.add_message("len-as-condition", node=node)
 
-    @utils.check_messages("len-as-condition")
-    def visit_compare(self, node):
-        # compare nodes are trickier because the len(S) expression
-        # may be somewhere in the middle of the node
-
-        # note: astroid.Compare has the left most operand in node.left
-        # while the rest are a list of tuples in node.ops
-        # the format of the tuple is ('compare operator sign', node)
-        # here we squash everything into `ops` to make it easier for processing later
-        ops = [("", node.left)]
-        ops.extend(node.ops)
-        ops = list(itertools.chain(*ops))
-
-        for ops_idx in range(len(ops) - 2):
-            op_1 = ops[ops_idx]
-            op_2 = ops[ops_idx + 1]
-            op_3 = ops[ops_idx + 2]
-            error_detected = False
-
-            # 0 ?? len()
-            if (
-                _is_constant_zero(op_1)
-                and op_2 in ["==", "!=", "<", ">="]
-                and _is_len_call(op_3)
-            ):
-                error_detected = True
-            # len() ?? 0
-            elif (
-                _is_len_call(op_1)
-                and op_2 in ["==", "!=", ">", "<="]
-                and _is_constant_zero(op_3)
-            ):
-                error_detected = True
-            elif (
-                _has_constant_value(op_1, value=1)
-                and op_2 == ">"
-                and _is_len_call(op_3)
-            ):
-                error_detected = True
-            elif (
-                _is_len_call(op_1)
-                and op_2 == "<"
-                and _has_constant_value(op_3, value=1)
-            ):
-                error_detected = True
-
-            if error_detected:
-                parent = node.parent
-                # traverse the AST to figure out if this comparison was part of
-                # a test condition
-                while parent and not _node_is_test_condition(parent):
-                    parent = parent.parent
-
-                # report only if this len() comparison is part of a test condition
-                # for example: return len() > 0 should not report anything
-                if _node_is_test_condition(parent):
-                    self.add_message("len-as-condition", node=node)
-
-
-def is_trailing_comma(tokens, index):
-    """Check if the given token is a trailing comma
-
-    :param tokens: Sequence of modules tokens
-    :type tokens: list[tokenize.TokenInfo]
-    :param int index: Index of token under check in tokens
-    :returns: True if the token is a comma which trails an expression
-    :rtype: bool
-    """
-    token = tokens[index]
-    if token.exact_type != tokenize.COMMA:
-        return False
-    # Must have remaining tokens on the same line such as NEWLINE
-    left_tokens = itertools.islice(tokens, index + 1, None)
-    same_line_remaining_tokens = list(
-        itertools.takewhile(
-            lambda other_token, _token=token: other_token.start[0] == _token.start[0],
-            left_tokens,
-        )
-    )
-    # Note: If the newline is tokenize.NEWLINE and not tokenize.NL
-    # then the newline denotes the end of expression
-    is_last_element = all(
-        other_token.type in (tokenize.NEWLINE, tokenize.COMMENT)
-        for other_token in same_line_remaining_tokens
-    )
-    if not same_line_remaining_tokens or not is_last_element:
-        return False
-
-    def get_curline_index_start():
-        """Get the index denoting the start of the current line"""
-        for subindex, token in enumerate(reversed(tokens[:index])):
-            # See Lib/tokenize.py and Lib/token.py in cpython for more info
-            if token.type in (tokenize.NEWLINE, tokenize.NL):
-                return index - subindex
-        return 0
-
-    curline_start = get_curline_index_start()
-    expected_tokens = {"return", "yield"}
-    for prevtoken in tokens[curline_start:index]:
-        if "=" in prevtoken.string or prevtoken.string in expected_tokens:
-            return True
-    return False
-
 
 def register(linter):
     """Required method to auto register this checker."""
