@@ -19,8 +19,12 @@
 """
 
 import os
-import tokenize
 import re
+import tokenize
+
+from pylint.checkers import BaseTokenChecker
+from pylint.checkers.utils import check_messages
+from pylint.interfaces import IAstroidChecker, ITokenChecker
 
 try:
     import enchant
@@ -43,10 +47,6 @@ except ImportError:
         pass
 
 
-from pylint.interfaces import ITokenChecker, IAstroidChecker
-from pylint.checkers import BaseTokenChecker
-from pylint.checkers.utils import check_messages
-
 if enchant is not None:
     br = enchant.Broker()
     dicts = br.list_dicts()
@@ -57,7 +57,7 @@ if enchant is not None:
 else:
     dicts = "none"
     dict_choices = [""]
-    instr = " To make it working install python-enchant package."
+    instr = " To make it work, install the python-enchant package."
 
 
 class WordsWithDigigtsFilter(Filter):
@@ -184,7 +184,7 @@ class SpellingChecker(BaseTokenChecker):
                 "metavar": "<dict name>",
                 "choices": dict_choices,
                 "help": "Spelling dictionary name. "
-                "Available dictionaries: %s.%s." % (dicts, instr),
+                "Available dictionaries: %s.%s" % (dicts, instr),
             },
         ),
         (
@@ -202,7 +202,7 @@ class SpellingChecker(BaseTokenChecker):
                 "default": "",
                 "type": "string",
                 "metavar": "<path to file>",
-                "help": "A path to a file that contains private "
+                "help": "A path to a file that contains the private "
                 "dictionary; one word per line.",
             },
         ),
@@ -212,9 +212,9 @@ class SpellingChecker(BaseTokenChecker):
                 "default": "n",
                 "type": "yn",
                 "metavar": "<y_or_n>",
-                "help": "Tells whether to store unknown words to "
-                "indicated private dictionary in "
-                "--spelling-private-dict-file option instead of "
+                "help": "Tells whether to store unknown words to the "
+                "private dictionary (see the "
+                "--spelling-private-dict-file option) instead of "
                 "raising a message.",
             },
         ),
@@ -284,12 +284,17 @@ class SpellingChecker(BaseTokenChecker):
 
     def _check_spelling(self, msgid, line, line_num):
         original_line = line
+        try:
+            initial_space = re.search(r"^[^\S]\s*", line).regs[0][1]
+        except (IndexError, AttributeError):
+            initial_space = 0
         if line.strip().startswith("#"):
             line = line.strip()[1:]
             starts_with_comment = True
         else:
             starts_with_comment = False
-        for word, _ in self.tokenizer(line.strip()):
+        for word, word_start_at in self.tokenizer(line.strip()):
+            word_start_at += initial_space
             lower_cased_word = word.casefold()
 
             # Skip words from ignore list.
@@ -328,12 +333,15 @@ class SpellingChecker(BaseTokenChecker):
                 suggestions = self.spelling_dict.suggest(word)
                 del suggestions[self.config.max_spelling_suggestions :]
 
-                m = re.search(r"(\W|^)(%s)(\W|$)" % word, line)
-                if m:
+                line_segment = line[word_start_at:]
+                match = re.search(r"(\W|^)(%s)(\W|$)" % word, line_segment)
+                if match:
                     # Start position of second group in regex.
-                    col = m.regs[2][0]
+                    col = match.regs[2][0]
                 else:
-                    col = line.index(word)
+                    col = line_segment.index(word)
+
+                col += word_start_at
 
                 if starts_with_comment:
                     col += 1
