@@ -26,10 +26,9 @@ import sys
 import astroid
 from astroid.bases import Instance
 from astroid.node_classes import Const
-from pylint.interfaces import IAstroidChecker
-from pylint.checkers import BaseChecker
-from pylint.checkers import utils
 
+from pylint.checkers import BaseChecker, utils
+from pylint.interfaces import IAstroidChecker
 
 OPEN_FILES = {"open", "file"}
 UNITTEST_CASE = "unittest.case"
@@ -38,11 +37,8 @@ COPY_COPY = "copy.copy"
 OS_ENVIRON = "os._Environ"
 ENV_GETTERS = {"os.getenv"}
 SUBPROCESS_POPEN = "subprocess.Popen"
-
-if sys.version_info >= (3, 0):
-    OPEN_MODULE = "_io"
-else:
-    OPEN_MODULE = "__builtin__"
+SUBPROCESS_RUN = "subprocess.run"
+OPEN_MODULE = "_io"
 
 
 def _check_mode_str(mode):
@@ -148,6 +144,13 @@ class StdlibChecker(BaseChecker):
             "trivial! Minimize the number of libraries you call into."
             "https://docs.python.org/3/library/subprocess.html#popen-constructor",
         ),
+        "W1510": (
+            "Using subprocess.run without explicitly set `check` is not recommended.",
+            "subprocess-run-check",
+            "The check parameter should always be used with explicitly set "
+            "`check` keyword to make clear what the error-handling behavior is."
+            "https://docs.python.org/3/library/subprocess.html#subprocess.runs",
+        ),
     }
 
     deprecated = {
@@ -245,12 +248,17 @@ class StdlibChecker(BaseChecker):
         if not node.kwargs and not node.keywords and len(node.args) <= 1:
             self.add_message("bad-thread-instantiation", node=node)
 
-    def _check_for_preexec_fn_in_Popen(self, node):
+    def _check_for_preexec_fn_in_popen(self, node):
         if node.keywords:
             for keyword in node.keywords:
                 if keyword.arg == "preexec_fn":
                     self.add_message("subprocess-popen-preexec-fn", node=node)
 
+    def _check_for_check_kw_in_run(self, node):
+        kwargs = {keyword.arg for keyword in (node.keywords or ())}
+        if "check" not in kwargs:
+            self.add_message("subprocess-run-check", node=node)
+
     def _check_shallow_copy_environ(self, node):
         arg = utils.get_argument_from_call(node, position=0)
         for inferred in arg.inferred():
@@ -267,6 +275,7 @@ class StdlibChecker(BaseChecker):
         "invalid-envvar-value",
         "invalid-envvar-default",
         "subprocess-popen-preexec-fn",
+        "subprocess-run-check",
     )
     def visit_call(self, node):
         """Visit a Call node."""
@@ -274,7 +283,7 @@ class StdlibChecker(BaseChecker):
             for inferred in node.func.infer():
                 if inferred is astroid.Uninferable:
                     continue
-                elif inferred.root().name == OPEN_MODULE:
+                if inferred.root().name == OPEN_MODULE:
                     if getattr(node.func, "name", None) in OPEN_FILES:
                         self._check_open_mode(node)
                 elif inferred.root().name == UNITTEST_CASE:
@@ -283,13 +292,15 @@ class StdlibChecker(BaseChecker):
                     if inferred.qname() == THREADING_THREAD:
                         self._check_bad_thread_instantiation(node)
                     elif inferred.qname() == SUBPROCESS_POPEN:
-                        self._check_for_preexec_fn_in_Popen(node)
+                        self._check_for_preexec_fn_in_popen(node)
                 elif isinstance(inferred, astroid.FunctionDef):
                     name = inferred.qname()
                     if name == COPY_COPY:
                         self._check_shallow_copy_environ(node)
                     elif name in ENV_GETTERS:
                         self._check_env_function(node, inferred)
+                    elif name == SUBPROCESS_RUN:
+                        self._check_for_check_kw_in_run(node)
                 self._check_deprecated_method(node, inferred)
         except astroid.InferenceError:
             return
@@ -357,14 +368,14 @@ class StdlibChecker(BaseChecker):
             )
 
     def _check_datetime(self, node):
-        """ Check that a datetime was infered.
+        """ Check that a datetime was inferred.
         If so, emit boolean-datetime warning.
         """
         try:
-            infered = next(node.infer())
+            inferred = next(node.infer())
         except astroid.InferenceError:
             return
-        if isinstance(infered, Instance) and infered.qname() == "datetime.time":
+        if isinstance(inferred, Instance) and inferred.qname() == "datetime.time":
             self.add_message("boolean-datetime", node=node)
 
     def _check_open_mode(self, node):
