@@ -1671,17 +1671,13 @@ class VariablesChecker(BaseChecker):
 
     def _store_type_annotation_node(self, type_annotation):
         """Given a type annotation, store all the name nodes it refers to"""
-        if (
-            isinstance(type_annotation, astroid.Name)
-            and type_annotation.name in TYPING_NAMES
-        ):
+        if isinstance(type_annotation, astroid.Name):
             self._type_annotation_names.append(type_annotation.name)
             return
 
         if not isinstance(type_annotation, astroid.Subscript):
             return
 
-        # Check if it is namespaced by typing or not.
         if (
             isinstance(type_annotation.value, astroid.Attribute)
             and isinstance(type_annotation.value.expr, astroid.Name)
@@ -1693,7 +1689,6 @@ class VariablesChecker(BaseChecker):
         self._type_annotation_names.extend(
             annotation.name
             for annotation in type_annotation.nodes_of_class(astroid.Name)
-            if annotation.name in TYPING_NAMES
         )
 
     def _store_type_annotation_names(self, node):
