@@ -52,23 +52,22 @@ For example:
 You may also use py_run to run pylint with desired options and get back (or not)
 its output.
 """
-from __future__ import print_function
-
 import os
 import os.path as osp
-import sys
 import shlex
-from subprocess import Popen, PIPE
+import sys
 from io import StringIO
+from subprocess import PIPE, Popen
 
 
 def _get_env():
-    '''Extracts the environment PYTHONPATH and appends the current sys.path to
-    those.'''
+    """Extracts the environment PYTHONPATH and appends the current sys.path to
+    those."""
     env = dict(os.environ)
-    env['PYTHONPATH'] = os.pathsep.join(sys.path)
+    env["PYTHONPATH"] = os.pathsep.join(sys.path)
     return env
 
+
 def lint(filename, options=()):
     """Pylint the given file.
 
@@ -90,18 +89,27 @@ def lint(filename, options=()):
     parent_path = osp.dirname(full_path)
     child_path = osp.basename(full_path)
 
-    while parent_path != "/" and osp.exists(osp.join(parent_path, '__init__.py')):
+    while parent_path != "/" and osp.exists(osp.join(parent_path, "__init__.py")):
         child_path = osp.join(osp.basename(parent_path), child_path)
         parent_path = osp.dirname(parent_path)
 
     # Start pylint
     # Ensure we use the python and pylint associated with the running epylint
     run_cmd = "import sys; from pylint.lint import Run; Run(sys.argv[1:])"
-    cmd = [sys.executable, "-c", run_cmd] + [
-        '--msg-template', '{path}:{line}: {category} ({msg_id}, {symbol}, {obj}) {msg}',
-        '-r', 'n', child_path] + list(options)
-    process = Popen(cmd, stdout=PIPE, cwd=parent_path, env=_get_env(),
-                    universal_newlines=True)
+    cmd = (
+        [sys.executable, "-c", run_cmd]
+        + [
+            "--msg-template",
+            "{path}:{line}: {category} ({msg_id}, {symbol}, {obj}) {msg}",
+            "-r",
+            "n",
+            child_path,
+        ]
+        + list(options)
+    )
+    process = Popen(
+        cmd, stdout=PIPE, cwd=parent_path, env=_get_env(), universal_newlines=True
+    )
 
     for line in process.stdout:
         # remove pylintrc warning
@@ -112,13 +120,13 @@ def lint(filename, options=()):
         parts = line.split(":")
         if parts and parts[0] == child_path:
             line = ":".join([filename] + parts[1:])
-        print(line, end=' ')
+        print(line, end=" ")
 
     process.wait()
     return process.returncode
 
 
-def py_run(command_options='', return_std=False, stdout=None, stderr=None):
+def py_run(command_options="", return_std=False, stdout=None, stderr=None):
     """Run pylint from python
 
     ``command_options`` is a string containing ``pylint`` command line options;
@@ -139,9 +147,12 @@ def py_run(command_options='', return_std=False, stdout=None, stderr=None):
     To silently run Pylint on a module, and get its standard output and error:
         >>> (pylint_stdout, pylint_stderr) = py_run( 'module_name.py', True)
     """
+    # Detect if we use Python as executable or not, else default to `python`
+    executable = sys.executable if "python" in sys.executable else "python"
+
     # Create command line to call pylint
-    epylint_part = [sys.executable, "-c", "from pylint import epylint;epylint.Run()"]
-    options = shlex.split(command_options, posix=not sys.platform.startswith('win'))
+    epylint_part = [executable, "-c", "from pylint import epylint;epylint.Run()"]
+    options = shlex.split(command_options, posix=not sys.platform.startswith("win"))
     cli = epylint_part + options
 
     # Providing standard output and/or error if not set
@@ -156,8 +167,14 @@ def py_run(command_options='', return_std=False, stdout=None, stderr=None):
         else:
             stderr = sys.stderr
     # Call pylint in a subprocess
-    process = Popen(cli, shell=False, stdout=stdout, stderr=stderr,
-                    env=_get_env(), universal_newlines=True)
+    process = Popen(
+        cli,
+        shell=False,
+        stdout=stdout,
+        stderr=stderr,
+        env=_get_env(),
+        universal_newlines=True,
+    )
     proc_stdout, proc_stderr = process.communicate()
     # Return standard output and error
     if return_std:
@@ -176,5 +193,5 @@ def Run():
         sys.exit(lint(sys.argv[1], sys.argv[2:]))
 
 
-if __name__ == '__main__':
+if __name__ == "__main__":
     Run()
