@@ -16,8 +16,6 @@
 
 """Utility methods for docstring checking."""
 
-from __future__ import absolute_import, print_function
-
 import re
 
 import astroid
@@ -230,7 +228,7 @@ class Docstring:
 
 class SphinxDocstring(Docstring):
     re_type = r"""
-        [~!]?                # Optional link style prefix
+        [~!.]?               # Optional link style prefix
         \w(?:\w|\.[^\.])*    # Valid python name
         """
 
@@ -443,7 +441,7 @@ class GoogleDocstring(Docstring):
 
     re_multiple_type = r"""
         (?:{container_type}|{type}|{xref})
-        (?:\s+or\s+(?:{container_type}|{type}|{xref}))*
+        (?:\s+(?:of|or)\s+(?:{container_type}|{type}|{xref}))*
     """.format(
         type=re_type, xref=re_xref, container_type=re_container_type
     )
