@@ -8,9 +8,10 @@
 # For details: https://github.com/PyCQA/pylint/blob/master/COPYING
 
 import astroid
+
 from pylint.checkers import BaseTokenChecker
 from pylint.checkers.utils import check_messages
-from pylint.interfaces import ITokenChecker, IAstroidChecker
+from pylint.interfaces import IAstroidChecker, ITokenChecker
 
 
 class ElseifUsedChecker(BaseTokenChecker):
@@ -18,13 +19,16 @@ class ElseifUsedChecker(BaseTokenChecker):
     """
 
     __implements__ = (ITokenChecker, IAstroidChecker)
-    name = 'else_if_used'
-    msgs = {'R5501': ('Consider using "elif" instead of "else if"',
-                      'else-if-used',
-                      'Used when an else statement is immediately followed by '
-                      'an if statement and does not contain statements that '
-                      'would be unrelated to it.'),
-           }
+    name = "else_if_used"
+    msgs = {
+        "R5501": (
+            'Consider using "elif" instead of "else if"',
+            "else-if-used",
+            "Used when an else statement is immediately followed by "
+            "an if statement and does not contain statements that "
+            "would be unrelated to it.",
+        )
+    }
 
     def __init__(self, linter=None):
         BaseTokenChecker.__init__(self, linter)
@@ -37,28 +41,30 @@ class ElseifUsedChecker(BaseTokenChecker):
     def process_tokens(self, tokens):
         # Process tokens and look for 'if' or 'elif'
         for _, token, _, _, _ in tokens:
-            if token == 'elif':
+            if token == "elif":
                 self._elifs.append(True)
-            elif token == 'if':
+            elif token == "if":
                 self._elifs.append(False)
 
     def leave_module(self, _):
         self._init()
 
-    def visit_ifexp(self, _):
+    def visit_ifexp(self, node):
+        if isinstance(node.parent, astroid.FormattedValue):
+            return
         self._if_counter += 1
 
     def visit_comprehension(self, node):
         self._if_counter += len(node.ifs)
 
-    @check_messages('else-if-used')
+    @check_messages("else-if-used")
     def visit_if(self, node):
         if isinstance(node.parent, astroid.If):
             orelse = node.parent.orelse
             # current if node must directly follow an "else"
             if orelse and orelse == [node]:
                 if not self._elifs[self._if_counter]:
-                    self.add_message('else-if-used', node=node)
+                    self.add_message("else-if-used", node=node)
         self._if_counter += 1
 
 
