@@ -46,49 +46,63 @@
 
 # pylint: disable=broad-except
 
-""" %prog [options] modules_or_packages
+""" pylint [options] modules_or_packages
 
   Check that module(s) satisfy a coding standard (and more !).
 
-    %prog --help
+    pylint --help
 
   Display this help message and exit.
 
-    %prog --help-msg <msg-id>[,<msg-id>]
+    pylint --help-msg <msg-id>[,<msg-id>]
 
   Display help messages about given message identifiers and exit.
 """
-from __future__ import print_function
-
 import collections
 import contextlib
 import operator
 import os
+import sys
+import tokenize
+import traceback
+import warnings
+from io import TextIOWrapper
+
+import astroid
+from astroid import modutils
+from astroid.__pkginfo__ import version as astroid_version
+from astroid.builder import AstroidBuilder
+
+from pylint import __pkginfo__, checkers, config, exceptions, interfaces, reporters
+from pylint.__pkginfo__ import version
+from pylint.constants import MAIN_CHECKER_NAME, MSG_TYPES, OPTION_RGX
+from pylint.message import Message, MessageDefinitionStore, MessagesHandlerMixIn
+from pylint.reporters.ureports import nodes as report_nodes
+from pylint.utils import ASTWalker, FileState, utils
 
 try:
     import multiprocessing
 except ImportError:
     multiprocessing = None  # type: ignore
-import sys
-import tokenize
-import warnings
-
-import astroid
-from astroid.__pkginfo__ import version as astroid_version
-from astroid import modutils
-from pylint import checkers
-from pylint import interfaces
-from pylint import reporters
-from pylint import exceptions
-from pylint import utils
-from pylint import config
-from pylint.__pkginfo__ import version
-from pylint.reporters.ureports import nodes as report_nodes
 
 
 MANAGER = astroid.MANAGER
 
 
+def _ast_from_string(data, filepath, modname):
+    cached = MANAGER.astroid_cache.get(modname)
+    if cached and cached.file == filepath:
+        return cached
+
+    return AstroidBuilder(MANAGER).string_build(data, modname, filepath)
+
+
+def _read_stdin():
+    # https://mail.python.org/pipermail/python-list/2012-November/634424.html
+    sys.stdin = TextIOWrapper(sys.stdin.detach(), encoding="utf-8")
+    return sys.stdin.read()
+
+
 def _get_new_args(message):
     location = (
         message.abspath,
@@ -288,11 +302,11 @@ if multiprocessing is not None:
             )
 
 
-# pylint: disable=too-many-instance-attributes
+# pylint: disable=too-many-instance-attributes,too-many-public-methods
 class PyLinter(
     config.OptionsManagerMixIn,
-    utils.MessagesHandlerMixIn,
-    utils.ReportsHandlerMixIn,
+    MessagesHandlerMixIn,
+    reporters.ReportsHandlerMixIn,
     checkers.BaseTokenChecker,
 ):
     """lint Python modules using external checkers.
@@ -310,7 +324,7 @@ class PyLinter(
 
     __implements__ = (interfaces.ITokenChecker,)
 
-    name = "master"
+    name = MAIN_CHECKER_NAME
     priority = 0
     level = 0
     msgs = MSGS
@@ -358,7 +372,7 @@ class PyLinter(
                     "default": (),
                     "level": 1,
                     "help": "List of plugins (as comma separated values of "
-                    "python modules names) to load, usually to register "
+                    "python module names) to load, usually to register "
                     "additional checkers.",
                 },
             ),
@@ -397,12 +411,12 @@ class PyLinter(
                     "level": 1,
                     "default": "10.0 - ((float(5 * error + warning + refactor + "
                     "convention) / statement) * 10)",
-                    "help": "Python expression which should return a note less "
-                    "than 10 (10 is the highest note). You have access "
-                    "to the variables errors warning, statement which "
-                    "respectively contain the number of errors / "
-                    "warnings messages and the total number of "
-                    "statements analyzed. This is used by the global "
+                    "help": "Python expression which should return a score less "
+                    "than or equal to 10. You have access to the variables "
+                    "'error', 'warning', 'refactor', and 'convention' which "
+                    "contain the number of messages in each category, as well as "
+                    "'statement' which is the total number of statements "
+                    "analyzed. This score is used by the global "
                     "evaluation report (RP0004).",
                 },
             ),
@@ -556,6 +570,16 @@ class PyLinter(
                     ),
                 },
             ),
+            (
+                "from-stdin",
+                {
+                    "action": "store_true",
+                    "help": (
+                        "Interpret the stdin as a python script, whose filename "
+                        "needs to be passed as the module_or_package argument."
+                    ),
+                },
+            ),
         )
 
     option_groups = (
@@ -567,7 +591,7 @@ class PyLinter(
         # some stuff has to be done before ancestors initialization...
         #
         # messages store / checkers / reporter / astroid manager
-        self.msgs_store = utils.MessagesStore()
+        self.msgs_store = MessageDefinitionStore()
         self.reporter = None
         self._reporter_name = None
         self._reporters = {}
@@ -575,7 +599,7 @@ class PyLinter(
         self._pragma_lineno = {}
         self._ignore_file = False
         # visit variables
-        self.file_state = utils.FileState()
+        self.file_state = FileState()
         self.current_name = None
         self.current_file = None
         self.stats = None
@@ -588,13 +612,13 @@ class PyLinter(
             "disable-msg": self.disable,
             "enable-msg": self.enable,
         }
-        full_version = "%%prog %s\nastroid %s\nPython %s" % (
+        full_version = "pylint %s\nastroid %s\nPython %s" % (
             version,
             astroid_version,
             sys.version,
         )
-        utils.MessagesHandlerMixIn.__init__(self)
-        utils.ReportsHandlerMixIn.__init__(self)
+        MessagesHandlerMixIn.__init__(self)
+        reporters.ReportsHandlerMixIn.__init__(self)
         super(PyLinter, self).__init__(
             usage=__doc__, version=full_version, config_file=pylintrc or config.PYLINTRC
         )
@@ -738,7 +762,6 @@ class PyLinter(
         checker.load_defaults()
 
         # Register the checker, but disable all of its messages.
-        # TODO(cpopa): we should have a better API for this.
         if not getattr(checker, "enabled", True):
             self.disable(checker.name)
 
@@ -797,6 +820,25 @@ class PyLinter(
             self.global_set_option("disable", value)
         self._python3_porting_mode = True
 
+    def list_messages_enabled(self):
+        enabled = [
+            "  %s (%s)" % (message.symbol, message.msgid)
+            for message in self.msgs_store.messages
+            if self.is_message_enabled(message.msgid)
+        ]
+        disabled = [
+            "  %s (%s)" % (message.symbol, message.msgid)
+            for message in self.msgs_store.messages
+            if not self.is_message_enabled(message.msgid)
+        ]
+        print("Enabled messages:")
+        for msg in sorted(enabled):
+            print(msg)
+        print("\nDisabled messages:")
+        for msg in sorted(disabled):
+            print(msg)
+        print("")
+
     # block level option handling #############################################
     #
     # see func_block_disable_msg.py test case for expected behaviour
@@ -806,10 +848,21 @@ class PyLinter(
         level options
         """
         control_pragmas = {"disable", "enable"}
+        prev_line = None
+        saw_newline = True
+        seen_newline = True
         for (tok_type, content, start, _, _) in tokens:
+            if prev_line and prev_line != start[0]:
+                saw_newline = seen_newline
+                seen_newline = False
+
+            prev_line = start[0]
+            if tok_type in (tokenize.NL, tokenize.NEWLINE):
+                seen_newline = True
+
             if tok_type != tokenize.COMMENT:
                 continue
-            match = utils.OPTION_RGX.search(content)
+            match = OPTION_RGX.search(content)
             if match is None:
                 continue
 
@@ -861,6 +914,10 @@ class PyLinter(
                             self.add_message("file-ignored", line=start[0])
                             self._ignore_file = True
                             return
+                        # If we did not see a newline between the previous line and now,
+                        # we saw a backslash so treat the two lines as one.
+                        if not saw_newline:
+                            meth(msgid, "module", start[0] - 1)
                         meth(msgid, "module", start[0])
                     except exceptions.UnknownMessageError:
                         self.add_message("bad-option-value", args=msgid, line=start[0])
@@ -882,7 +939,11 @@ class PyLinter(
         """Get all the checker names that this linter knows about."""
         current_checkers = self.get_checkers()
         return sorted(
-            {check.name for check in current_checkers if check.name != "master"}
+            {
+                checker.name
+                for checker in current_checkers
+                if checker.name != MAIN_CHECKER_NAME
+            }
         )
 
     def prepare_checkers(self):
@@ -890,16 +951,16 @@ class PyLinter(
         if not self.config.reports:
             self.disable_reporters()
         # get needed checkers
-        neededcheckers = [self]
+        needed_checkers = [self]
         for checker in self.get_checkers()[1:]:
             messages = {msg for msg in checker.msgs if self.is_message_enabled(msg)}
             if messages or any(self.report_is_enabled(r[0]) for r in checker.reports):
-                neededcheckers.append(checker)
+                needed_checkers.append(checker)
         # Sort checkers by priority
-        neededcheckers = sorted(
-            neededcheckers, key=operator.attrgetter("priority"), reverse=True
+        needed_checkers = sorted(
+            needed_checkers, key=operator.attrgetter("priority"), reverse=True
         )
-        return neededcheckers
+        return needed_checkers
 
     # pylint: disable=unused-argument
     @staticmethod
@@ -1022,7 +1083,7 @@ class PyLinter(
             (_, self.file_state.base_name, module, messages, stats, msg_status) = result
 
             for msg in messages:
-                msg = utils.Message(*msg)
+                msg = Message(*msg)
                 self.set_current_module(module)
                 self.reporter.handle_message(msg)
 
@@ -1038,7 +1099,7 @@ class PyLinter(
                 checker.stats = self.stats
 
     def _do_check(self, files_or_modules):
-        walker = utils.PyLintASTWalker(self)
+        walker = ASTWalker(self)
         _checkers = self.prepare_checkers()
         tokencheckers = [
             c
@@ -1054,31 +1115,63 @@ class PyLinter(
             if interfaces.implements(checker, interfaces.IAstroidChecker):
                 walker.add_checker(checker)
         # build ast and check modules or packages
-        for descr in self.expand_files(files_or_modules):
-            modname, filepath, is_arg = descr["name"], descr["path"], descr["isarg"]
-            if not self.should_analyze_file(modname, filepath, is_argument=is_arg):
-                continue
+        if self.config.from_stdin:
+            if len(files_or_modules) != 1:
+                raise exceptions.InvalidArgsError(
+                    "Missing filename required for --from-stdin"
+                )
+
+            filepath = files_or_modules[0]
+            try:
+                # Note that this function does not really perform an
+                # __import__ but may raise an ImportError exception, which
+                # we want to catch here.
+                modname = ".".join(modutils.modpath_from_file(filepath))
+            except ImportError:
+                modname = os.path.splitext(os.path.basename(filepath))[0]
 
             self.set_current_module(modname, filepath)
+
             # get the module representation
-            ast_node = self.get_ast(filepath, modname)
-            if ast_node is None:
-                continue
-            # XXX to be correct we need to keep module_msgs_state for every
-            # analyzed module (the problem stands with localized messages which
-            # are only detected in the .close step)
-            self.file_state = utils.FileState(descr["basename"])
-            self._ignore_file = False
-            # fix the current file (if the source file was not available or
-            # if it's actually a c extension)
-            self.current_file = ast_node.file  # pylint: disable=maybe-no-member
-            self.check_astroid_module(ast_node, walker, rawcheckers, tokencheckers)
-            # warn about spurious inline messages handling
-            spurious_messages = self.file_state.iter_spurious_suppression_messages(
-                self.msgs_store
-            )
-            for msgid, line, args in spurious_messages:
-                self.add_message(msgid, line, None, args)
+            ast_node = _ast_from_string(_read_stdin(), filepath, modname)
+
+            if ast_node is not None:
+                self.file_state = FileState(filepath)
+                self.check_astroid_module(ast_node, walker, rawcheckers, tokencheckers)
+                # warn about spurious inline messages handling
+                spurious_messages = self.file_state.iter_spurious_suppression_messages(
+                    self.msgs_store
+                )
+                for msgid, line, args in spurious_messages:
+                    self.add_message(msgid, line, None, args)
+        else:
+            for descr in self.expand_files(files_or_modules):
+                modname, filepath, is_arg = descr["name"], descr["path"], descr["isarg"]
+                if not self.should_analyze_file(modname, filepath, is_argument=is_arg):
+                    continue
+
+                self.set_current_module(modname, filepath)
+                # get the module representation
+                ast_node = self.get_ast(filepath, modname)
+                if ast_node is None:
+                    continue
+
+                self.file_state = FileState(descr["basename"])
+                self._ignore_file = False
+                # fix the current file (if the source file was not available or
+                # if it's actually a c extension)
+                self.current_file = ast_node.file  # pylint: disable=maybe-no-member
+                before_check_statements = walker.nbstatements
+                self.check_astroid_module(ast_node, walker, rawcheckers, tokencheckers)
+                self.stats["by_module"][modname]["statement"] = (
+                    walker.nbstatements - before_check_statements
+                )
+                # warn about spurious inline messages handling
+                spurious_messages = self.file_state.iter_spurious_suppression_messages(
+                    self.msgs_store
+                )
+                for msgid, line, args in spurious_messages:
+                    self.add_message(msgid, line, None, args)
         # notify global end
         self.stats["statement"] = walker.nbstatements
         for checker in reversed(_checkers):
@@ -1110,7 +1203,7 @@ class PyLinter(
         self.current_file = filepath or modname
         self.stats["by_module"][modname] = {}
         self.stats["by_module"][modname]["statement"] = 0
-        for msg_cat in utils.MSG_TYPES.values():
+        for msg_cat in MSG_TYPES.values():
             self.stats["by_module"][modname][msg_cat] = 0
 
     def get_ast(self, filepath, modname):
@@ -1120,13 +1213,14 @@ class PyLinter(
         except astroid.AstroidSyntaxError as ex:
             # pylint: disable=no-member
             self.add_message(
-                "syntax-error", line=getattr(ex.error, "lineno", 0), args=str(ex.error)
+                "syntax-error",
+                line=getattr(ex.error, "lineno", 0),
+                col_offset=getattr(ex.error, "offset", None),
+                args=str(ex.error),
             )
         except astroid.AstroidBuildingException as ex:
             self.add_message("parse-error", args=ex)
         except Exception as ex:
-            import traceback
-
             traceback.print_exc()
             self.add_message("astroid-error", args=(ex.__class__, ex))
 
@@ -1166,7 +1260,7 @@ class PyLinter(
         MANAGER.always_load_extensions = self.config.unsafe_load_any_extension
         MANAGER.max_inferable_values = self.config.limit_inference_results
         MANAGER.extension_package_whitelist.update(self.config.extension_pkg_whitelist)
-        for msg_cat in utils.MSG_TYPES.values():
+        for msg_cat in MSG_TYPES.values():
             self.stats[msg_cat] = 0
 
     def generate_reports(self):
@@ -1180,7 +1274,6 @@ class PyLinter(
         if self.file_state.base_name is not None:
             # load previous results if any
             previous_stats = config.load_results(self.file_state.base_name)
-            # XXX code below needs refactoring to be more reporter agnostic
             self.reporter.on_close(self.stats, previous_stats)
             if self.config.reports:
                 sect = self.make_reports(self.stats, previous_stats)
@@ -1349,9 +1442,7 @@ def fix_import_path(args):
     changes = []
     for arg in args:
         path = _get_python_path(arg)
-        if path in changes:
-            continue
-        else:
+        if path not in changes:
             changes.append(path)
     sys.path[:] = changes + ["."] + sys.path
     try:
@@ -1441,6 +1532,18 @@ group are mutually exclusive.",
                         "help": "Generate pylint's messages.",
                     },
                 ),
+                (
+                    "list-msgs-enabled",
+                    {
+                        "action": "callback",
+                        "metavar": "<msg-id>",
+                        "callback": self.cb_list_messages_enabled,
+                        "group": "Commands",
+                        "level": 1,
+                        "help": "Display a list of what messages are enabled "
+                        "and disabled with the given configuration.",
+                    },
+                ),
                 (
                     "list-groups",
                     {
@@ -1658,8 +1761,6 @@ group are mutually exclusive.",
 
     def cb_generate_manpage(self, *args, **kwargs):
         """optik callback for sample config file generation"""
-        from pylint import __pkginfo__
-
         self.linter.generate_manpage(__pkginfo__)
         sys.exit(0)
 
@@ -1673,11 +1774,16 @@ group are mutually exclusive.",
         self.linter.print_full_documentation()
         sys.exit(0)
 
-    def cb_list_messages(self, option, optname, value, parser):  # FIXME
+    def cb_list_messages(self, option, optname, value, parser):
         """optik callback for printing available messages"""
         self.linter.msgs_store.list_messages()
         sys.exit(0)
 
+    def cb_list_messages_enabled(self, option, optname, value, parser):
+        """optik callback for printing available messages"""
+        self.linter.list_messages_enabled()
+        sys.exit(0)
+
     def cb_list_groups(self, *args, **kwargs):
         """List all the check groups that pylint knows about
 
