@@ -13,16 +13,14 @@
 
   create UML diagrams for classes and modules in <packages>
 """
-from __future__ import print_function
-
 import os
 import subprocess
 import sys
 
 from pylint.config import ConfigurationMixIn
-from pylint.pyreverse.inspector import Linker, project_from_files
-from pylint.pyreverse.diadefslib import DiadefsHandler
 from pylint.pyreverse import writer
+from pylint.pyreverse.diadefslib import DiadefsHandler
+from pylint.pyreverse.inspector import Linker, project_from_files
 from pylint.pyreverse.utils import insert_default_options
 
 OPTIONS = (
@@ -113,11 +111,6 @@ OPTIONS = (
             help="include module name in representation of classes",
         ),
     ),
-    # TODO : generate dependencies like in pylint
-    # ("package-dependencies",
-    # dict(short="M", action="store", metavar='<package_depth>', type='int',
-    #     help='show <package_depth> module dependencies beyond modules in \
-    # <projects> (for the package diagram)')),
     (
         "only-classnames",
         dict(
